@@ -18,17 +18,27 @@
 
 import json
 import logging
+import pathlib
+import subprocess
 import typing
 import unittest.mock
 
+import _pytest.capture
 import pytest
 
 import switchbot_mqtt
+import switchbot_mqtt._cli
 
 # pylint: disable=protected-access; tests
 # pylint: disable=too-many-arguments; these are tests, no API
 
 
+def test_console_entry_point() -> None:
+    assert subprocess.run(
+        ["switchbot-mqtt", "--help"], stdout=subprocess.PIPE, check=True
+    ).stdout.startswith(b"usage: ")
+
+
 @pytest.mark.parametrize(
     (
         "argv",
@@ -84,22 +94,23 @@ import switchbot_mqtt
     ],
 )
 def test__main(
-    argv,
-    expected_mqtt_host,
-    expected_mqtt_port,
-    expected_username,
-    expected_password,
-    expected_retry_count,
-):
+    argv: typing.List[str],
+    expected_mqtt_host: str,
+    expected_mqtt_port: int,
+    expected_username: str,
+    expected_password: str,
+    expected_retry_count: int,
+) -> None:
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv", argv
-    ):
-        switchbot_mqtt._main()
+    ), pytest.warns(UserWarning, match=r"Please add --mqtt-disable-tls\b"):
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(
         mqtt_host=expected_mqtt_host,
         mqtt_port=expected_mqtt_port,
         mqtt_username=expected_username,
         mqtt_password=expected_password,
+        mqtt_disable_tls=True,
         retry_count=expected_retry_count,
         device_passwords={},
         fetch_device_info=False,
@@ -121,11 +132,10 @@ def test__main(
     ],
 )
 def test__main_mqtt_password_file(
-    tmpdir, mqtt_password_file_content, expected_password
-):
-    mqtt_password_path = tmpdir.join("mqtt-password")
-    with mqtt_password_path.open("w") as mqtt_password_file:
-        mqtt_password_file.write(mqtt_password_file_content)
+    tmp_path: pathlib.Path, mqtt_password_file_content: str, expected_password: str
+) -> None:
+    mqtt_password_path = tmp_path.joinpath("mqtt-password")
+    mqtt_password_path.write_text(mqtt_password_file_content, encoding="utf8")
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv",
         [
@@ -138,19 +148,22 @@ def test__main_mqtt_password_file(
             str(mqtt_password_path),
         ],
     ):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(
         mqtt_host="localhost",
         mqtt_port=1883,
         mqtt_username="me",
         mqtt_password=expected_password,
+        mqtt_disable_tls=True,
         retry_count=3,
         device_passwords={},
         fetch_device_info=False,
     )
 
 
-def test__main_mqtt_password_file_collision(capsys):
+def test__main_mqtt_password_file_collision(
+    capsys: _pytest.capture.CaptureFixture,
+) -> None:
     with unittest.mock.patch(
         "sys.argv",
         [
@@ -166,7 +179,7 @@ def test__main_mqtt_password_file_collision(capsys):
         ],
     ):
         with pytest.raises(SystemExit):
-            switchbot_mqtt._main()
+            switchbot_mqtt._cli._main()
     out, err = capsys.readouterr()
     assert not out
     assert (
@@ -182,8 +195,10 @@ def test__main_mqtt_password_file_collision(capsys):
         {"11:22:33:44:55:66": "password", "aa:bb:cc:dd:ee:ff": "secret"},
     ],
 )
-def test__main_device_password_file(tmpdir, device_passwords):
-    device_passwords_path = tmpdir.join("passwords.json")
+def test__main_device_password_file(
+    tmp_path: pathlib.Path, device_passwords: typing.Dict[str, str]
+) -> None:
+    device_passwords_path = tmp_path.joinpath("passwords.json")
     device_passwords_path.write_text(json.dumps(device_passwords), encoding="utf8")
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv",
@@ -195,19 +210,80 @@ def test__main_device_password_file(tmpdir, device_passwords):
             str(device_passwords_path),
         ],
     ):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(
         mqtt_host="localhost",
         mqtt_port=1883,
         mqtt_username=None,
         mqtt_password=None,
+        mqtt_disable_tls=True,
         retry_count=3,
         device_passwords=device_passwords,
         fetch_device_info=False,
     )
 
 
-def test__main_fetch_device_info():
+_RUN_DEFAULT_KWARGS: typing.Dict[str, typing.Any] = {
+    "mqtt_port": 1883,
+    "mqtt_username": None,
+    "mqtt_password": None,
+    "mqtt_disable_tls": False,
+    "retry_count": 3,
+    "device_passwords": {},
+    "fetch_device_info": False,
+}
+
+
+def test__main_mqtt_disable_tls_implicit() -> None:
+    with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
+        "sys.argv", ["", "--mqtt-host", "mqtt.local"]
+    ), pytest.warns(UserWarning, match=r"Please add --mqtt-disable-tls\b"):
+        switchbot_mqtt._cli._main()
+    run_mock.assert_called_once_with(
+        **{
+            **_RUN_DEFAULT_KWARGS,
+            "mqtt_host": "mqtt.local",
+            "mqtt_disable_tls": True,
+            "mqtt_port": 1883,
+        }
+    )
+
+
+def test__main_mqtt_enable_tls() -> None:
+    with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
+        "sys.argv", ["", "--mqtt-host", "mqtt.local", "--mqtt-enable-tls"]
+    ):
+        switchbot_mqtt._cli._main()
+    run_mock.assert_called_once_with(
+        **{**_RUN_DEFAULT_KWARGS, "mqtt_host": "mqtt.local", "mqtt_port": 8883}
+    )
+
+
+def test__main_mqtt_enable_tls_overwrite_port() -> None:
+    with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
+        "sys.argv",
+        ["", "--mqtt-host", "mqtt.local", "--mqtt-port", "1883", "--mqtt-enable-tls"],
+    ):
+        switchbot_mqtt._cli._main()
+    run_mock.assert_called_once_with(
+        **{**_RUN_DEFAULT_KWARGS, "mqtt_host": "mqtt.local", "mqtt_port": 1883}
+    )
+
+
+def test__main_mqtt_tls_collision(capsys: _pytest.capture.CaptureFixture) -> None:
+    with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
+        "sys.argv",
+        ["", "--mqtt-host", "mqtt.local", "--mqtt-enable-tls", "--mqtt-disable-tls"],
+    ), pytest.raises(SystemExit):
+        switchbot_mqtt._cli._main()
+    run_mock.assert_not_called()
+    assert (
+        "error: argument --mqtt-disable-tls: not allowed with argument --mqtt-enable-tls\n"
+        in capsys.readouterr()[1]
+    )
+
+
+def test__main_fetch_device_info() -> None:
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv",
         [
@@ -216,12 +292,13 @@ def test__main_fetch_device_info():
             "localhost",
         ],
     ):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     default_kwargs = dict(
         mqtt_host="localhost",
         mqtt_port=1883,
         mqtt_username=None,
         mqtt_password=None,
+        mqtt_disable_tls=True,
         retry_count=3,
         device_passwords={},
     )
@@ -230,25 +307,25 @@ def test__main_fetch_device_info():
         "sys.argv",
         ["", "--mqtt-host", "localhost", "--fetch-device-info"],
     ):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(fetch_device_info=True, **default_kwargs)
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv",
         ["", "--mqtt-host", "localhost"],
     ), unittest.mock.patch.dict("os.environ", {"FETCH_DEVICE_INFO": "21"}):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(fetch_device_info=True, **default_kwargs)
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv",
         ["", "--mqtt-host", "localhost"],
     ), unittest.mock.patch.dict("os.environ", {"FETCH_DEVICE_INFO": ""}):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(fetch_device_info=False, **default_kwargs)
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv",
         ["", "--mqtt-host", "localhost"],
     ), unittest.mock.patch.dict("os.environ", {"FETCH_DEVICE_INFO": " "}):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     run_mock.assert_called_once_with(fetch_device_info=True, **default_kwargs)
 
 
@@ -265,7 +342,7 @@ def test__main_fetch_device_info():
 )
 def test__main_log_config(
     additional_argv: typing.List[str], root_log_level: int, log_format: str
-):
+) -> None:
     with unittest.mock.patch(
         "sys.argv", ["", "--mqtt-host", "localhost"] + additional_argv
     ), unittest.mock.patch(
@@ -273,7 +350,7 @@ def test__main_log_config(
     ) as logging_basic_config_mock, unittest.mock.patch(
         "switchbot_mqtt._run"
     ):
-        switchbot_mqtt._main()
+        switchbot_mqtt._cli._main()
     logging_basic_config_mock.assert_called_once_with(
         level=root_log_level, format=log_format, datefmt="%Y-%m-%dT%H:%M:%S%z"
     )
