@@ -20,10 +20,15 @@ import logging
 import typing
 import unittest.mock
 
+import _pytest.logging
 import pytest
 from paho.mqtt.client import MQTT_ERR_QUEUE_SIZE, MQTT_ERR_SUCCESS, MQTTMessage, Client
 
 import switchbot_mqtt
+import switchbot_mqtt._actors
+from switchbot_mqtt._actors import _ButtonAutomator, _CurtainMotor
+from switchbot_mqtt._actors._base import _MQTTCallbackUserdata, _MQTTControlledActor
+from switchbot_mqtt._utils import _MQTTTopicLevel, _MQTTTopicPlaceholder
 
 # pylint: disable=protected-access
 # pylint: disable=too-many-arguments; these are tests, no API
@@ -38,8 +43,13 @@ import switchbot_mqtt
 )
 @pytest.mark.parametrize("fetch_device_info", [True, False])
 def test__run(
-    caplog, mqtt_host, mqtt_port, retry_count, device_passwords, fetch_device_info
-):
+    caplog: _pytest.logging.LogCaptureFixture,
+    mqtt_host: str,
+    mqtt_port: int,
+    retry_count: int,
+    device_passwords: typing.Dict[str, str],
+    fetch_device_info: bool,
+) -> None:
     with unittest.mock.patch(
         "paho.mqtt.client.Client"
     ) as mqtt_client_mock, caplog.at_level(logging.DEBUG):
@@ -48,78 +58,125 @@ def test__run(
             mqtt_port=mqtt_port,
             mqtt_username=None,
             mqtt_password=None,
+            mqtt_disable_tls=False,
             retry_count=retry_count,
             device_passwords=device_passwords,
             fetch_device_info=fetch_device_info,
         )
-    mqtt_client_mock.assert_called_once_with(
-        userdata=switchbot_mqtt._MQTTCallbackUserdata(
-            retry_count=retry_count,
-            device_passwords=device_passwords,
-            fetch_device_info=fetch_device_info,
-        )
+    mqtt_client_mock.assert_called_once()
+    assert not mqtt_client_mock.call_args[0]
+    assert set(mqtt_client_mock.call_args[1].keys()) == {"userdata"}
+    userdata = mqtt_client_mock.call_args[1]["userdata"]
+    assert userdata == _MQTTCallbackUserdata(
+        retry_count=retry_count,
+        device_passwords=device_passwords,
+        fetch_device_info=fetch_device_info,
     )
     assert not mqtt_client_mock().username_pw_set.called
+    mqtt_client_mock().tls_set.assert_called_once_with(ca_certs=None)
     mqtt_client_mock().connect.assert_called_once_with(host=mqtt_host, port=mqtt_port)
     mqtt_client_mock().socket().getpeername.return_value = (mqtt_host, mqtt_port)
     with caplog.at_level(logging.DEBUG):
-        mqtt_client_mock().on_connect(mqtt_client_mock(), None, {}, 0)
-    assert mqtt_client_mock().subscribe.call_args_list == [
-        unittest.mock.call("homeassistant/switch/switchbot/+/set"),
-        unittest.mock.call("homeassistant/cover/switchbot-curtain/+/set"),
-    ]
-    assert mqtt_client_mock().message_callback_add.call_args_list == [
-        unittest.mock.call(
-            sub="homeassistant/switch/switchbot/+/set",
-            callback=switchbot_mqtt._ButtonAutomator._mqtt_command_callback,
-        ),
-        unittest.mock.call(
-            sub="homeassistant/cover/switchbot-curtain/+/set",
-            callback=switchbot_mqtt._CurtainMotor._mqtt_command_callback,
-        ),
-    ]
+        mqtt_client_mock().on_connect(mqtt_client_mock(), userdata, {}, 0)
+    subscribe_mock = mqtt_client_mock().subscribe
+    assert subscribe_mock.call_count == (5 if fetch_device_info else 3)
+    for topic in [
+        "homeassistant/switch/switchbot/+/set",
+        "homeassistant/cover/switchbot-curtain/+/set",
+        "homeassistant/cover/switchbot-curtain/+/position/set-percent",
+    ]:
+        assert unittest.mock.call(topic) in subscribe_mock.call_args_list
+    for topic in [
+        "homeassistant/switch/switchbot/+/request-device-info",
+        "homeassistant/cover/switchbot-curtain/+/request-device-info",
+    ]:
+        assert (
+            unittest.mock.call(topic) in subscribe_mock.call_args_list
+        ) == fetch_device_info
+    callbacks = {
+        c[1]["sub"]: c[1]["callback"]
+        for c in mqtt_client_mock().message_callback_add.call_args_list
+    }
+    assert (  # pylint: disable=comparison-with-callable; intended
+        callbacks["homeassistant/cover/switchbot-curtain/+/position/set-percent"]
+        == _CurtainMotor._mqtt_set_position_callback
+    )
     mqtt_client_mock().loop_forever.assert_called_once_with()
-    assert caplog.record_tuples == [
+    assert caplog.record_tuples[:2] == [
         (
             "switchbot_mqtt",
             logging.INFO,
-            "connecting to MQTT broker {}:{}".format(mqtt_host, mqtt_port),
+            f"connecting to MQTT broker {mqtt_host}:{mqtt_port} (TLS enabled)",
         ),
         (
             "switchbot_mqtt",
             logging.DEBUG,
-            "connected to MQTT broker {}:{}".format(mqtt_host, mqtt_port),
-        ),
-        (
-            "switchbot_mqtt",
-            logging.INFO,
-            "subscribing to MQTT topic 'homeassistant/switch/switchbot/+/set'",
-        ),
-        (
-            "switchbot_mqtt",
-            logging.INFO,
-            "subscribing to MQTT topic 'homeassistant/cover/switchbot-curtain/+/set'",
+            f"connected to MQTT broker {mqtt_host}:{mqtt_port}",
         ),
     ]
+    assert len(caplog.record_tuples) == (7 if fetch_device_info else 5)
+    assert (
+        "switchbot_mqtt._actors._base",
+        logging.INFO,
+        "subscribing to MQTT topic 'homeassistant/switch/switchbot/+/set'",
+    ) in caplog.record_tuples
+    assert (
+        "switchbot_mqtt._actors._base",
+        logging.INFO,
+        "subscribing to MQTT topic 'homeassistant/cover/switchbot-curtain/+/set'",
+    ) in caplog.record_tuples
+
+
+@pytest.mark.parametrize("mqtt_disable_tls", [True, False])
+def test__run_tls(
+    caplog: _pytest.logging.LogCaptureFixture, mqtt_disable_tls: bool
+) -> None:
+    with unittest.mock.patch(
+        "paho.mqtt.client.Client"
+    ) as mqtt_client_mock, caplog.at_level(logging.INFO):
+        switchbot_mqtt._run(
+            mqtt_host="mqtt.local",
+            mqtt_port=1234,
+            mqtt_username=None,
+            mqtt_password=None,
+            mqtt_disable_tls=mqtt_disable_tls,
+            retry_count=21,
+            device_passwords={},
+            fetch_device_info=True,
+        )
+    if mqtt_disable_tls:
+        mqtt_client_mock().tls_set.assert_not_called()
+    else:
+        mqtt_client_mock().tls_set.assert_called_once_with(ca_certs=None)
+    if mqtt_disable_tls:
+        assert caplog.record_tuples[0][2].endswith(" (TLS disabled)")
+    else:
+        assert caplog.record_tuples[0][2].endswith(" (TLS enabled)")
 
 
 @pytest.mark.parametrize("mqtt_host", ["mqtt-broker.local"])
 @pytest.mark.parametrize("mqtt_port", [1833])
 @pytest.mark.parametrize("mqtt_username", ["me"])
 @pytest.mark.parametrize("mqtt_password", [None, "secret"])
-def test__run_authentication(mqtt_host, mqtt_port, mqtt_username, mqtt_password):
+def test__run_authentication(
+    mqtt_host: str,
+    mqtt_port: int,
+    mqtt_username: str,
+    mqtt_password: typing.Optional[str],
+) -> None:
     with unittest.mock.patch("paho.mqtt.client.Client") as mqtt_client_mock:
         switchbot_mqtt._run(
             mqtt_host=mqtt_host,
             mqtt_port=mqtt_port,
             mqtt_username=mqtt_username,
             mqtt_password=mqtt_password,
+            mqtt_disable_tls=True,
             retry_count=7,
             device_passwords={},
             fetch_device_info=True,
         )
     mqtt_client_mock.assert_called_once_with(
-        userdata=switchbot_mqtt._MQTTCallbackUserdata(
+        userdata=_MQTTCallbackUserdata(
             retry_count=7, device_passwords={}, fetch_device_info=True
         )
     )
@@ -131,7 +188,9 @@ def test__run_authentication(mqtt_host, mqtt_port, mqtt_username, mqtt_password)
 @pytest.mark.parametrize("mqtt_host", ["mqtt-broker.local"])
 @pytest.mark.parametrize("mqtt_port", [1833])
 @pytest.mark.parametrize("mqtt_password", ["secret"])
-def test__run_authentication_missing_username(mqtt_host, mqtt_port, mqtt_password):
+def test__run_authentication_missing_username(
+    mqtt_host: str, mqtt_port: int, mqtt_password: str
+) -> None:
     with unittest.mock.patch("paho.mqtt.client.Client"):
         with pytest.raises(ValueError):
             switchbot_mqtt._run(
@@ -139,6 +198,7 @@ def test__run_authentication_missing_username(mqtt_host, mqtt_port, mqtt_passwor
                 mqtt_port=mqtt_port,
                 mqtt_username=None,
                 mqtt_password=mqtt_password,
+                mqtt_disable_tls=True,
                 retry_count=3,
                 device_passwords={},
                 fetch_device_info=True,
@@ -146,12 +206,17 @@ def test__run_authentication_missing_username(mqtt_host, mqtt_port, mqtt_passwor
 
 
 def _mock_actor_class(
-    command_topic_levels: typing.List[switchbot_mqtt._MQTTTopicLevel],
+    *,
+    command_topic_levels: typing.Tuple[_MQTTTopicLevel, ...] = NotImplemented,
+    request_info_levels: typing.Tuple[_MQTTTopicLevel, ...] = NotImplemented,
 ) -> typing.Type:
-    class _ActorMock(switchbot_mqtt._MQTTControlledActor):
+    class _ActorMock(_MQTTControlledActor):
         MQTT_COMMAND_TOPIC_LEVELS = command_topic_levels
+        _MQTT_UPDATE_DEVICE_INFO_TOPIC_LEVELS = request_info_levels
 
-        def __init__(self, mac_address, retry_count, password):
+        def __init__(
+            self, mac_address: str, retry_count: int, password: typing.Optional[str]
+        ) -> None:
             super().__init__(
                 mac_address=mac_address, retry_count=retry_count, password=password
             )
@@ -161,53 +226,140 @@ def _mock_actor_class(
             mqtt_message_payload: bytes,
             mqtt_client: Client,
             update_device_info: bool,
-        ):
+        ) -> None:
             pass
 
+        def _get_device(self) -> None:
+            return None
+
     return _ActorMock
 
 
+@pytest.mark.parametrize(
+    ("topic_levels", "topic", "expected_mac_address"),
+    [
+        (
+            switchbot_mqtt._actors._ButtonAutomator._MQTT_UPDATE_DEVICE_INFO_TOPIC_LEVELS,
+            b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/request-device-info",
+            "aa:bb:cc:dd:ee:ff",
+        ),
+    ],
+)
+@pytest.mark.parametrize("payload", [b"", b"whatever"])
+def test__mqtt_update_device_info_callback(
+    caplog: _pytest.logging.LogCaptureFixture,
+    topic_levels: typing.Tuple[_MQTTTopicLevel, ...],
+    topic: bytes,
+    expected_mac_address: str,
+    payload: bytes,
+) -> None:
+    ActorMock = _mock_actor_class(request_info_levels=topic_levels)
+    message = MQTTMessage(topic=topic)
+    message.payload = payload
+    callback_userdata = _MQTTCallbackUserdata(
+        retry_count=21,  # tested in test__mqtt_command_callback
+        device_passwords={},
+        fetch_device_info=True,
+    )
+    with unittest.mock.patch.object(
+        ActorMock, "__init__", return_value=None
+    ) as init_mock, unittest.mock.patch.object(
+        ActorMock, "_update_and_report_device_info"
+    ) as update_mock, caplog.at_level(
+        logging.DEBUG
+    ):
+        ActorMock._mqtt_update_device_info_callback(
+            "client_dummy", callback_userdata, message
+        )
+    init_mock.assert_called_once_with(
+        mac_address=expected_mac_address, retry_count=21, password=None
+    )
+    update_mock.assert_called_once_with("client_dummy")
+    assert caplog.record_tuples == [
+        (
+            "switchbot_mqtt._actors._base",
+            logging.DEBUG,
+            f"received topic={topic.decode()} payload={payload!r}",
+        )
+    ]
+
+
+def test__mqtt_update_device_info_callback_ignore_retained(
+    caplog: _pytest.logging.LogCaptureFixture,
+) -> None:
+    ActorMock = _mock_actor_class(
+        request_info_levels=(_MQTTTopicPlaceholder.MAC_ADDRESS, "request")
+    )
+    message = MQTTMessage(topic=b"aa:bb:cc:dd:ee:ff/request")
+    message.payload = b""
+    message.retain = True
+    with unittest.mock.patch.object(
+        ActorMock, "__init__", return_value=None
+    ) as init_mock, unittest.mock.patch.object(
+        ActorMock, "execute_command"
+    ) as execute_command_mock, caplog.at_level(
+        logging.DEBUG
+    ):
+        ActorMock._mqtt_update_device_info_callback(
+            "client_dummy",
+            _MQTTCallbackUserdata(
+                retry_count=21, device_passwords={}, fetch_device_info=True
+            ),
+            message,
+        )
+    init_mock.assert_not_called()
+    execute_command_mock.assert_not_called()
+    assert caplog.record_tuples == [
+        (
+            "switchbot_mqtt._actors._base",
+            logging.DEBUG,
+            "received topic=aa:bb:cc:dd:ee:ff/request payload=b''",
+        ),
+        ("switchbot_mqtt._actors._base", logging.INFO, "ignoring retained message"),
+    ]
+
+
 @pytest.mark.parametrize(
     ("command_topic_levels", "topic", "payload", "expected_mac_address"),
     [
         (
-            switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
+            _ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
             b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/set",
             b"ON",
             "aa:bb:cc:dd:ee:ff",
         ),
         (
-            switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
+            _ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
             b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/set",
             b"OFF",
             "aa:bb:cc:dd:ee:ff",
         ),
         (
-            switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
+            _ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
             b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/set",
             b"on",
             "aa:bb:cc:dd:ee:ff",
         ),
         (
-            switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
+            _ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
             b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/set",
             b"off",
             "aa:bb:cc:dd:ee:ff",
         ),
         (
-            switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
+            _ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS,
             b"homeassistant/switch/switchbot/aa:01:23:45:67:89/set",
             b"ON",
             "aa:01:23:45:67:89",
         ),
         (
-            ["switchbot", switchbot_mqtt._MQTTTopicPlaceholder.MAC_ADDRESS],
+            ["switchbot", _MQTTTopicPlaceholder.MAC_ADDRESS],
             b"switchbot/aa:01:23:45:67:89",
             b"ON",
             "aa:01:23:45:67:89",
         ),
         (
-            switchbot_mqtt._CurtainMotor.MQTT_COMMAND_TOPIC_LEVELS,
+            _CurtainMotor.MQTT_COMMAND_TOPIC_LEVELS,
             b"homeassistant/cover/switchbot-curtain/aa:01:23:45:67:89/set",
             b"OPEN",
             "aa:01:23:45:67:89",
@@ -217,18 +369,18 @@ def _mock_actor_class(
 @pytest.mark.parametrize("retry_count", (3, 42))
 @pytest.mark.parametrize("fetch_device_info", [True, False])
 def test__mqtt_command_callback(
-    caplog,
-    command_topic_levels: typing.List[switchbot_mqtt._MQTTTopicLevel],
+    caplog: _pytest.logging.LogCaptureFixture,
+    command_topic_levels: typing.Tuple[_MQTTTopicLevel, ...],
     topic: bytes,
     payload: bytes,
     expected_mac_address: str,
     retry_count: int,
     fetch_device_info: bool,
-):
-    ActorMock = _mock_actor_class(command_topic_levels)
+) -> None:
+    ActorMock = _mock_actor_class(command_topic_levels=command_topic_levels)
     message = MQTTMessage(topic=topic)
     message.payload = payload
-    callback_userdata = switchbot_mqtt._MQTTCallbackUserdata(
+    callback_userdata = _MQTTCallbackUserdata(
         retry_count=retry_count,
         device_passwords={},
         fetch_device_info=fetch_device_info,
@@ -251,9 +403,9 @@ def test__mqtt_command_callback(
     )
     assert caplog.record_tuples == [
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors._base",
             logging.DEBUG,
-            "received topic={} payload={!r}".format(topic.decode(), payload),
+            f"received topic={topic.decode()} payload={payload!r}",
         )
     ]
 
@@ -266,16 +418,15 @@ def test__mqtt_command_callback(
         ("11:22:33:dd:ee:ff", "äöü"),
     ],
 )
-def test__mqtt_command_callback_password(mac_address, expected_password):
+def test__mqtt_command_callback_password(
+    mac_address: str, expected_password: typing.Optional[str]
+) -> None:
     ActorMock = _mock_actor_class(
-        [
-            "switchbot",
-            switchbot_mqtt._MQTTTopicPlaceholder.MAC_ADDRESS,
-        ]
+        command_topic_levels=("switchbot", _MQTTTopicPlaceholder.MAC_ADDRESS)
     )
     message = MQTTMessage(topic=b"switchbot/" + mac_address.encode())
     message.payload = b"whatever"
-    callback_userdata = switchbot_mqtt._MQTTCallbackUserdata(
+    callback_userdata = _MQTTCallbackUserdata(
         retry_count=3,
         device_passwords={
             "11:22:33:44:55:77": "test",
@@ -308,9 +459,11 @@ def test__mqtt_command_callback_password(mac_address, expected_password):
         (b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/set/suffix", b"ON"),
     ],
 )
-def test__mqtt_command_callback_unexpected_topic(caplog, topic: bytes, payload: bytes):
+def test__mqtt_command_callback_unexpected_topic(
+    caplog: _pytest.logging.LogCaptureFixture, topic: bytes, payload: bytes
+) -> None:
     ActorMock = _mock_actor_class(
-        switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS
+        command_topic_levels=_ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS
     )
     message = MQTTMessage(topic=topic)
     message.payload = payload
@@ -323,7 +476,7 @@ def test__mqtt_command_callback_unexpected_topic(caplog, topic: bytes, payload:
     ):
         ActorMock._mqtt_command_callback(
             "client_dummy",
-            switchbot_mqtt._MQTTCallbackUserdata(
+            _MQTTCallbackUserdata(
                 retry_count=3, device_passwords={}, fetch_device_info=True
             ),
             message,
@@ -332,26 +485,26 @@ def test__mqtt_command_callback_unexpected_topic(caplog, topic: bytes, payload:
     execute_command_mock.assert_not_called()
     assert caplog.record_tuples == [
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors._base",
             logging.DEBUG,
-            "received topic={} payload={!r}".format(topic.decode(), payload),
+            f"received topic={topic.decode()} payload={payload!r}",
         ),
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors._base",
             logging.WARNING,
-            "unexpected topic {}".format(topic.decode()),
+            f"unexpected topic {topic.decode()}",
         ),
     ]
 
 
 @pytest.mark.parametrize(("mac_address", "payload"), [("aa:01:23:4E:RR:OR", b"ON")])
 def test__mqtt_command_callback_invalid_mac_address(
-    caplog, mac_address: str, payload: bytes
-):
+    caplog: _pytest.logging.LogCaptureFixture, mac_address: str, payload: bytes
+) -> None:
     ActorMock = _mock_actor_class(
-        switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS
+        command_topic_levels=_ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS
     )
-    topic = "homeassistant/switch/switchbot/{}/set".format(mac_address).encode()
+    topic = f"homeassistant/switch/switchbot/{mac_address}/set".encode()
     message = MQTTMessage(topic=topic)
     message.payload = payload
     with unittest.mock.patch.object(
@@ -363,7 +516,7 @@ def test__mqtt_command_callback_invalid_mac_address(
     ):
         ActorMock._mqtt_command_callback(
             "client_dummy",
-            switchbot_mqtt._MQTTCallbackUserdata(
+            _MQTTCallbackUserdata(
                 retry_count=3, device_passwords={}, fetch_device_info=True
             ),
             message,
@@ -372,14 +525,14 @@ def test__mqtt_command_callback_invalid_mac_address(
     execute_command_mock.assert_not_called()
     assert caplog.record_tuples == [
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors._base",
             logging.DEBUG,
-            "received topic={} payload={!r}".format(topic.decode(), payload),
+            f"received topic={topic.decode()} payload={payload!r}",
         ),
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors._base",
             logging.WARNING,
-            "invalid mac address {}".format(mac_address),
+            f"invalid mac address {mac_address}",
         ),
     ]
 
@@ -388,9 +541,11 @@ def test__mqtt_command_callback_invalid_mac_address(
     ("topic", "payload"),
     [(b"homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/set", b"ON")],
 )
-def test__mqtt_command_callback_ignore_retained(caplog, topic: bytes, payload: bytes):
+def test__mqtt_command_callback_ignore_retained(
+    caplog: _pytest.logging.LogCaptureFixture, topic: bytes, payload: bytes
+) -> None:
     ActorMock = _mock_actor_class(
-        switchbot_mqtt._ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS
+        command_topic_levels=_ButtonAutomator.MQTT_COMMAND_TOPIC_LEVELS
     )
     message = MQTTMessage(topic=topic)
     message.payload = payload
@@ -404,7 +559,7 @@ def test__mqtt_command_callback_ignore_retained(caplog, topic: bytes, payload: b
     ):
         ActorMock._mqtt_command_callback(
             "client_dummy",
-            switchbot_mqtt._MQTTCallbackUserdata(
+            _MQTTCallbackUserdata(
                 retry_count=4, device_passwords={}, fetch_device_info=True
             ),
             message,
@@ -413,11 +568,11 @@ def test__mqtt_command_callback_ignore_retained(caplog, topic: bytes, payload: b
     execute_command_mock.assert_not_called()
     assert caplog.record_tuples == [
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors._base",
             logging.DEBUG,
-            "received topic={} payload={!r}".format(topic.decode(), payload),
+            f"received topic={topic.decode()} payload={payload!r}",
         ),
-        ("switchbot_mqtt", logging.INFO, "ignoring retained message"),
+        ("switchbot_mqtt._actors._base", logging.INFO, "ignoring retained message"),
     ]
 
 
@@ -426,12 +581,12 @@ def test__mqtt_command_callback_ignore_retained(caplog, topic: bytes, payload: b
     # https://www.home-assistant.io/docs/mqtt/discovery/#switches
     [
         (
-            switchbot_mqtt._ButtonAutomator.MQTT_STATE_TOPIC_LEVELS,
+            _ButtonAutomator.MQTT_STATE_TOPIC_LEVELS,
             "aa:bb:cc:dd:ee:ff",
             "homeassistant/switch/switchbot/aa:bb:cc:dd:ee:ff/state",
         ),
         (
-            ["switchbot", switchbot_mqtt._MQTTTopicPlaceholder.MAC_ADDRESS, "state"],
+            ["switchbot", _MQTTTopicPlaceholder.MAC_ADDRESS, "state"],
             "aa:bb:cc:dd:ee:gg",
             "switchbot/aa:bb:cc:dd:ee:gg/state",
         ),
@@ -440,18 +595,20 @@ def test__mqtt_command_callback_ignore_retained(caplog, topic: bytes, payload: b
 @pytest.mark.parametrize("state", [b"ON", b"CLOSE"])
 @pytest.mark.parametrize("return_code", [MQTT_ERR_SUCCESS, MQTT_ERR_QUEUE_SIZE])
 def test__report_state(
-    caplog,
-    state_topic_levels: typing.List[switchbot_mqtt._MQTTTopicLevel],
+    caplog: _pytest.logging.LogCaptureFixture,
+    state_topic_levels: typing.Tuple[_MQTTTopicLevel, ...],
     mac_address: str,
     expected_topic: str,
     state: bytes,
     return_code: int,
-):
+) -> None:
     # pylint: disable=too-many-arguments
-    class _ActorMock(switchbot_mqtt._MQTTControlledActor):
+    class _ActorMock(_MQTTControlledActor):
         MQTT_STATE_TOPIC_LEVELS = state_topic_levels
 
-        def __init__(self, mac_address, retry_count, password):
+        def __init__(
+            self, mac_address: str, retry_count: int, password: typing.Optional[str]
+        ) -> None:
             super().__init__(
                 mac_address=mac_address, retry_count=retry_count, password=password
             )
@@ -461,9 +618,12 @@ def test__report_state(
             mqtt_message_payload: bytes,
             mqtt_client: Client,
             update_device_info: bool,
-        ):
+        ) -> None:
             pass
 
+        def _get_device(self) -> None:
+            return None
+
     mqtt_client_mock = unittest.mock.MagicMock()
     mqtt_client_mock.publish.return_value.rc = return_code
     with caplog.at_level(logging.DEBUG):
@@ -473,19 +633,17 @@ def test__report_state(
         topic=expected_topic, payload=state, retain=True
     )
     assert caplog.record_tuples[0] == (
-        "switchbot_mqtt",
+        "switchbot_mqtt._actors._base",
         logging.DEBUG,
-        "publishing topic={} payload={!r}".format(expected_topic, state),
+        f"publishing topic={expected_topic} payload={state!r}",
     )
     if return_code == MQTT_ERR_SUCCESS:
         assert not caplog.records[1:]
     else:
         assert caplog.record_tuples[1:] == [
             (
-                "switchbot_mqtt",
+                "switchbot_mqtt._actors._base",
                 logging.ERROR,
-                "Failed to publish MQTT message on topic {} (rc={})".format(
-                    expected_topic, return_code
-                ),
+                f"Failed to publish MQTT message on topic {expected_topic} (rc={return_code})",
             )
         ]
