@@ -17,23 +17,23 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import logging
+import typing
 import unittest.mock
 
+import _pytest.logging
 import bluepy.btle
 import pytest
 
-import switchbot_mqtt
+from switchbot_mqtt._actors import _ButtonAutomator
 
 # pylint: disable=protected-access
 # pylint: disable=too-many-arguments; these are tests, no API
 
 
 @pytest.mark.parametrize("mac_address", ["{MAC_ADDRESS}", "aa:bb:cc:dd:ee:ff"])
-def test_get_mqtt_battery_percentage_topic(mac_address):
+def test_get_mqtt_battery_percentage_topic(mac_address: str) -> None:
     assert (
-        switchbot_mqtt._ButtonAutomator.get_mqtt_battery_percentage_topic(
-            mac_address=mac_address
-        )
+        _ButtonAutomator.get_mqtt_battery_percentage_topic(mac_address=mac_address)
         == f"homeassistant/switch/switchbot/{mac_address}/battery-percentage"
     )
 
@@ -41,24 +41,19 @@ def test_get_mqtt_battery_percentage_topic(mac_address):
 @pytest.mark.parametrize(("battery_percent", "battery_percent_encoded"), [(42, b"42")])
 def test__update_and_report_device_info(
     battery_percent: int, battery_percent_encoded: bytes
-):
+) -> None:
     with unittest.mock.patch("switchbot.SwitchbotCurtain.__init__", return_value=None):
-        actor = switchbot_mqtt._ButtonAutomator(
-            mac_address="dummy", retry_count=21, password=None
-        )
-    actor._get_device()._battery_percent = battery_percent
+        actor = _ButtonAutomator(mac_address="dummy", retry_count=21, password=None)
+    actor._get_device()._switchbot_device_data = {"data": {"battery": battery_percent}}
     mqtt_client_mock = unittest.mock.MagicMock()
     with unittest.mock.patch("switchbot.Switchbot.update") as update_mock:
         actor._update_and_report_device_info(mqtt_client=mqtt_client_mock)
     update_mock.assert_called_once_with()
-    assert mqtt_client_mock.publish.call_args_list == [
-        unittest.mock.call(topic=t, payload=battery_percent_encoded, retain=True)
-        for t in [
-            "homeassistant/switch/switchbot/dummy/battery-percentage",
-            # will be removed in v3:
-            "homeassistant/cover/switchbot/dummy/battery-percentage",
-        ]
-    ]
+    mqtt_client_mock.publish.assert_called_once_with(
+        topic="homeassistant/switch/switchbot/dummy/battery-percentage",
+        payload=battery_percent_encoded,
+        retain=True,
+    )
 
 
 @pytest.mark.parametrize("mac_address", ["aa:bb:cc:dd:ee:ff", "aa:bb:cc:11:22:33"])
@@ -78,19 +73,19 @@ def test__update_and_report_device_info(
 @pytest.mark.parametrize("update_device_info", [True, False])
 @pytest.mark.parametrize("command_successful", [True, False])
 def test_execute_command(
-    caplog,
-    mac_address,
-    password,
-    retry_count,
-    message_payload,
-    action_name,
-    update_device_info,
-    command_successful,
-):
+    caplog: _pytest.logging.LogCaptureFixture,
+    mac_address: str,
+    password: typing.Optional[str],
+    retry_count: int,
+    message_payload: bytes,
+    action_name: str,
+    update_device_info: bool,
+    command_successful: bool,
+) -> None:
     with unittest.mock.patch(
         "switchbot.Switchbot.__init__", return_value=None
     ) as device_init_mock, caplog.at_level(logging.INFO):
-        actor = switchbot_mqtt._ButtonAutomator(
+        actor = _ButtonAutomator(
             mac_address=mac_address, retry_count=retry_count, password=password
         )
         with unittest.mock.patch.object(
@@ -112,7 +107,7 @@ def test_execute_command(
     if command_successful:
         assert caplog.record_tuples == [
             (
-                "switchbot_mqtt",
+                "switchbot_mqtt._actors",
                 logging.INFO,
                 f"switchbot {mac_address} turned {message_payload.decode().lower()}",
             )
@@ -124,7 +119,7 @@ def test_execute_command(
     else:
         assert caplog.record_tuples == [
             (
-                "switchbot_mqtt",
+                "switchbot_mqtt._actors",
                 logging.ERROR,
                 f"failed to turn {message_payload.decode().lower()} switchbot {mac_address}",
             )
@@ -135,13 +130,13 @@ def test_execute_command(
 
 @pytest.mark.parametrize("mac_address", ["aa:bb:cc:dd:ee:ff"])
 @pytest.mark.parametrize("message_payload", [b"EIN", b""])
-def test_execute_command_invalid_payload(caplog, mac_address, message_payload):
+def test_execute_command_invalid_payload(
+    caplog: _pytest.logging.LogCaptureFixture, mac_address: str, message_payload: bytes
+) -> None:
     with unittest.mock.patch("switchbot.Switchbot") as device_mock, caplog.at_level(
         logging.INFO
     ):
-        actor = switchbot_mqtt._ButtonAutomator(
-            mac_address=mac_address, retry_count=21, password=None
-        )
+        actor = _ButtonAutomator(mac_address=mac_address, retry_count=21, password=None)
         with unittest.mock.patch.object(actor, "report_state") as report_mock:
             actor.execute_command(
                 mqtt_client="dummy",
@@ -153,7 +148,7 @@ def test_execute_command_invalid_payload(caplog, mac_address, message_payload):
     report_mock.assert_not_called()
     assert caplog.record_tuples == [
         (
-            "switchbot_mqtt",
+            "switchbot_mqtt._actors",
             logging.WARNING,
             f"unexpected payload {message_payload!r} (expected 'ON' or 'OFF')",
         )
@@ -162,7 +157,9 @@ def test_execute_command_invalid_payload(caplog, mac_address, message_payload):
 
 @pytest.mark.parametrize("mac_address", ["aa:bb:cc:dd:ee:ff"])
 @pytest.mark.parametrize("message_payload", [b"ON", b"OFF"])
-def test_execute_command_bluetooth_error(caplog, mac_address, message_payload):
+def test_execute_command_bluetooth_error(
+    caplog: _pytest.logging.LogCaptureFixture, mac_address: str, message_payload: bytes
+) -> None:
     """
     paho.mqtt.python>=1.5.1 no longer implicitly suppresses exceptions in callbacks.
     verify pySwitchbot catches exceptions raised in bluetooth stack.
@@ -175,22 +172,22 @@ def test_execute_command_bluetooth_error(caplog, mac_address, message_payload):
             f"Failed to connect to peripheral {mac_address}, addr type: random"
         ),
     ), caplog.at_level(logging.ERROR):
-        switchbot_mqtt._ButtonAutomator(
-            mac_address=mac_address, retry_count=3, password=None
+        _ButtonAutomator(
+            mac_address=mac_address, retry_count=0, password=None
         ).execute_command(
             mqtt_client="dummy",
             mqtt_message_payload=message_payload,
             update_device_info=True,
         )
-    assert caplog.record_tuples == [
-        (
-            "switchbot",
-            logging.ERROR,
-            "Switchbot communication failed. Stopping trying.",
-        ),
-        (
-            "switchbot_mqtt",
-            logging.ERROR,
-            f"failed to turn {message_payload.decode().lower()} switchbot {mac_address}",
-        ),
-    ]
+    assert len(caplog.records) == 2
+    assert caplog.records[0].name == "switchbot"
+    assert caplog.records[0].levelno == logging.ERROR
+    assert caplog.records[0].msg.startswith(
+        # pySwitchbot<0.11 had '.' suffix
+        "Switchbot communication failed. Stopping trying",
+    )
+    assert caplog.record_tuples[1] == (
+        "switchbot_mqtt._actors",
+        logging.ERROR,
+        f"failed to turn {message_payload.decode().lower()} switchbot {mac_address}",
+    )
